@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2011-2021 Canonical
- * Copyright (C) 2021-2023 Colin Ian King
+ * Copyright (C) 2021-2024 Colin Ian King
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -134,6 +134,7 @@ typedef struct timer_info {
 	double		last_used;	/* Last referenced */
 	double		prev_used;	/* Previous time used */
 	bool		kernel_thread;	/* True if task is a kernel thread */
+	bool		just_added;	/* True if recently added */
 } timer_info_t;
 
 typedef struct timer_stat {
@@ -281,7 +282,7 @@ static int pid_max_digits(void)
 
 	buf[n] = '\0';
 	max_digits = 0;
-	while ((buf[max_digits] >= '0') && (buf[max_digits] <= '9'))
+	while ((max_digits < n) && (buf[max_digits] >= '0') && (buf[max_digits] <= '9'))
 		max_digits++;
 	if (max_digits < min_digits)
 		max_digits = min_digits;
@@ -990,7 +991,7 @@ static void samples_dump(const char *filename)
 	}
 	(void)fprintf(fp, "\n");
 
-	(void)fprintf(fp, ",Init Function:");
+	(void)fprintf(fp, ",Kernel Init Function:");
 	for (i = 0; i < n; i++)
 		(void)fprintf(fp, ",%s", sorted_timer_infos[i]->func);
 	(void)fprintf(fp, "\n");
@@ -1534,25 +1535,30 @@ static OPTIMIZE3 void timer_stat_dump(
 				es_printf(" %16.16s", "Timer ID");
 			}
 			es_printf("%-*.*s", fn_size, fn_size,
-				" Init Function");
+				" Kernel Init Function");
 		}
 		es_printf("\n");
 
 		while (sorted) {
-			if (((n_lines == -1) || (j < n_lines)) &&
-			    (sorted->info->delta_events != 0)) {
+			uint64_t events;
+
+			if (g_opt_flags & OPT_CUMULATIVE)
+				events = sorted->info->total_events;
+			else
+				events = sorted->info->delta_events;
+
+			if (((n_lines == -1) || (j < n_lines)) && (events != 0)) {
 				char *task = (g_opt_flags & OPT_CMD) ?
 					sorted->info->cmdline :
 					sorted->info->comm;
 				j++;
 				if (g_opt_flags & OPT_CUMULATIVE) {
 					es_printf("%*" PRIu64 " ",
-						EVENTS_WIDTH,
-						sorted->info->total_events);
+						EVENTS_WIDTH, events);
 				} else {
 					es_printf("%*.2f ",
 						EVENTS_WIDTH,
-						(duration > 0.0) ? (double)sorted->info->delta_events / duration : 0.0);
+						(duration > 0.0) ? (double)events / duration : 0.0);
 				}
 
 				if (g_opt_flags & OPT_BRIEF) {
@@ -1730,11 +1736,17 @@ static void get_events(
 	const size_t app_name_len = strlen(g_app_name);
 	const double time_end = time_now + duration - 0.05;
 	char *tmpptr;
+	timer_info_t *pinfo;
 
 	tmpptr = read_events(time_end, get_events_buf);
 	if (!tmpptr)
 		return;
 
+	if (g_opt_flags & OPT_CUMULATIVE) {
+		for (pinfo = g_timer_info_list; pinfo; pinfo = pinfo->next)
+			pinfo->just_added = false;
+	}
+
 	while (*tmpptr) {
 		char *ptr, *eol = tmpptr;
 		char func[64], task[64];
@@ -1807,6 +1819,7 @@ static void get_events(
 			info.time_total = 0.0;
 			info.total_events = 1;
 			info.ident = make_hash_ident(&info);
+			info.just_added = true;
 			timer_stat_add(timer_stats, &info, time_now, duration);
 		}
 free_next:
@@ -1815,6 +1828,13 @@ free_next:
 next:
 		tmpptr = eol;
 	}
+
+	if (g_opt_flags & OPT_CUMULATIVE) {
+		for (pinfo = g_timer_info_list; pinfo; pinfo = pinfo->next) {
+			if (!pinfo->just_added)
+				timer_stat_add(timer_stats, pinfo, time_now, duration);
+		}
+	}
 }
 
 /*
